{ Demo program for ChroMathsome - Linear Algebra.
  Below, you find example implementations of using functions available in
  ChroMathsome - Linear Algebra.
  As an example, we will save and reload a bitmap picture.
  The picture will be stored using a matrix and a vector.
  Note:
  In this demonstration I've omitted to do any compressing of the file when it is saved.
  Maybe some smart guys out there can take this code as a starting point to create a "new"
  file saving algorithm.
  Note that this code can also be used to save a matrix to a file!

  Note that, since ChroMathsome has its own pre-defined exceptions, you do not need to
  write supersofisticated exception handling routines.
  The only exceptions written here are to make sure the pointers get freed, even
  when an exception has occured (a try ... finally block).

  Read the help file for information about the implemented functions
  and their limitations/possibilities!

  Author: John Merrit (john.merrit@net4all.be)
  Release date: January 30, 1998
  Reader may do with this code whatever he/she wants (give credits where credits are due ...).

  THIS SOFTWARE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS OR IMPLIED
  WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED WARRANTIES OF
  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.

  Brought to you by the letter "O".              }
unit Graphs;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  LinAlg, Menus, ExtCtrls;

type
  TForm1 = class(TForm)
    Image1: TImage;
    MainMenu1: TMainMenu;
    Close1: TMenuItem;
    File1: TMenuItem;
    Save1: TMenuItem;
    Open1: TMenuItem;
    procedure Close1Click(Sender: TObject);
    procedure Save1Click(Sender: TObject);
    procedure Open1Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    { Private declarations }
    function ColorPosInVector(Vec: RealVector; Cl: longint): integer;
  public
    { Public declarations }
  end;

var
  Form1: TForm1;
  V: RealVector;

implementation

{$R *.DFM}

procedure TForm1.Close1Click(Sender: TObject);
begin
     Close;
end;

function TForm1.ColorPosInVector(Vec: RealVector; Cl: longint): integer;
var i: longint;
    match: boolean;
begin
     match:= false;
     i:= 0;
     repeat
     begin
     i:= i + 1;
     if Round(Vec.Vector^[i]) = Cl then match:= true;
     end;
     until ((i = Vec.Dimension) or (match = true));
     if match = true then ColorPosInVector:= i
     else ColorPosInVector:= 0;
end;

{ Write the picture to a file.
  The colors are stored in a vector (V) while the matrix elements represent the dots of the picture.
  Every matrix element has a value that represents the position of the respective color of the
  respective dot in V.
  This works quite well for 16 color images.
  256, 64k, ..., colors -> Try to improve the code.                                           }
procedure TForm1.Save1Click(Sender: TObject);
var M: RealMatrix;
    i, j, k, first: integer;
    s: string;
    LColor: TColor;
    Li: longint;
    F: TextFile; // For the sake of simplicity, we use a text file.
begin
     InitRealMatrix(M, 1, 1);

     try
     first:= 0;  // V is initialized to have a dimension of 1.
     Redimension_RealMatrix(M, Image1.Width, Image1.Height);
     for i:= 1 to Image1.Width do
     for j:= 1 to Image1.Height do
     { Store the colors of the picture. }
     begin
       LColor:= Image1.Canvas.Pixels[i, j];
       Li:= ColorToRGB(LColor);

       if first = 0 then
       begin
         V.Vector^[1]:= Li;
         k:= 0;
         first:= 1
       end
       else
       begin // If V already contains the color ...
        k:= ColorPosInVector(V, Li);
        if k = 0 then RealVector_AddElement(V, Li); // If not -> add it!
       end;
       if k > 0 then M.Matrix^[i]^[j]:= k
       else M.Matrix^[i]^[j]:= V.Dimension;
     end;

     { Store matrix M and vector V to file DUMMY.TXT. }
     AssignFile(F, 'DUMMY.TXT');
     Rewrite(F);
     writeln(F, IntToStr(M.Row));
     writeln(F, IntToStr(M.Col));
     writeln(F, IntToStr(V.Dimension));
     for i:= 1 to V.Dimension do
     writeln(F, FloatToStr(V.Vector^[i]));

     for i:= 1 to M.Row do
     begin
     s:= '';
     for j:= 1 to M.Col do
     begin
     s:= s + FloatToStr(M.Matrix^[i]^[j]) + ' ';
     end;
     writeln(F, s);
     end;

     CloseFile(F);

     finally
     DestroyRealMatrix(M);
     end;
end;

{ Load (= read) the picture from a file and reconstruct it.           }
procedure TForm1.Open1Click(Sender: TObject);
var M: RealMatrix;
    i, j, k: integer;
    s, s1: string;
    LColor: TColor;
    Li: longint;
    F: TextFile;
begin
     if FileExists('DUMMY.TXT') = false then
     MessageDlg('You must save the image first!', mtError, [mbOK], 0)
     else
     begin
     InitRealMatrix(M, 1, 1);

     try
     AssignFile(F, 'DUMMY.TXT');
     reset(F);
     readln(F, s);  // Read dimensions of M.
     i:= StrToInt(s);
     readln(F, s);
     j:= StrToInt(s);
     Redimension_RealMatrix(M, i, j);
     readln(F, s); // Read dimension of V.
     i:= StrToInt(s);
     Redimension_RealVector(V, i);

     { Construct V. }
     for i:= 1 to V.Dimension do
     begin
       readln(F, s);
       V.Vector^[i]:= StrToFloat(s);
     end;

     { Construct matrix M. }
     for i:= 1 to M.Row do
     begin
       readln(F, s);
       { Strip s to retrieve matrix numbers. }
       for j:= 1 to M.Col - 1 do
       begin
         s1:= copy(s, 1, pos(' ', s) - 1);
         delete(s, 1, Length(s1) + 1);
         M.Matrix^[i]^[j]:= StrToFloat(s1);
       end;
     end;
     CloseFile(F);

     { Draw the image on Form1's Canvas. }
     for i:= 1 to M.Row do
     for j:= 1 to M.Col do
     begin
       k:= Round(M.Matrix^[i]^[j]); // The values of M and V are of type extended ...
       Form1.Canvas.Pixels[50 + i, 200 + j]:= Round(V.Vector^[k]);
     end;

     finally
     DestroyRealMatrix(M);
     end;
     end;
end;

procedure TForm1.FormCreate(Sender: TObject);
begin
     InitRealVector(V, 1);
     V.Vector^[1]:= -1;
end;

procedure TForm1.FormDestroy(Sender: TObject);
begin
     DestroyRealVector(V);
end;

end.
